----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch
--
-- Module Name: scalp_aurora_phy - arch
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.2
-- Description: scalp_aurora_phy
--
-- Last update: 2020-11-25
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.axi4_pkg.all;
use work.aurora_status_pkg.all;
use work.aurora_drp_pkg.all;

entity scalp_aurora_phy is

    generic (
        C_DEBUG_MODE                 : boolean := false;
        C_RX_FIFO_MODE               : boolean := true;
        C_RX_FIFO_RST_DONE_DLY_TICKS : integer := 100);

    port (
        -- Clocks
        -- GTP Ref Clocks
        GTRefClkxCI               : in  t_gt_ref_slave_clk;
        -- Aurora System and GTP Clocks
        AuroraClkxCI              : in  t_aurora_slave_clk;
        AuroraClkxCO              : out t_aurora_master_clk;
        -- Reset
        -- Aurora Reset
        AuroraResetxRI            : in  t_aurora_slave_reset;
        AuroraResetxRO            : out t_aurora_master_link_reset;
        -- RX Fifo and Back Pressure Reset
        RXResetxRI                : in  t_rx_reset;
        RXFifoResetDonexDO        : out t_rx_fifo_reset_done;
        RXFifoResetDoneDelayedxDO : out t_rx_fifo_reset_done;
        -- Back Pressure Reset
        -- GTP Serial IO
        -- North
        GTPFromNorthxDI           : in  t_aurora_gtp_diff_io_rx;
        GTPToNorthxDO             : out t_aurora_gtp_diff_io_tx;
        -- East
        GTPFromEastxDI            : in  t_aurora_gtp_diff_io_rx;
        GTPToEastxDO              : out t_aurora_gtp_diff_io_tx;
        -- South
        GTPFromSouthxDI           : in  t_aurora_gtp_diff_io_rx;
        GTPToSouthxDO             : out t_aurora_gtp_diff_io_tx;
        -- West
        GTPFromWestxDI            : in  t_aurora_gtp_diff_io_rx;
        GTPToWestxDO              : out t_aurora_gtp_diff_io_tx;
        -- Axi4 Framing Interface
        -- North
        NorthRXM2SxDO             : out t_axi4m2s;
        NorthRXS2MxDI             : in  t_axi4s2m;
        NorthTXM2SxDI             : in  t_axi4m2s;
        NorthTXS2MxDO             : out t_axi4s2m;
        -- East
        EastRXM2SxDO              : out t_axi4m2s;
        EastRXS2MxDI              : in  t_axi4s2m;
        EastTXM2SxDI              : in  t_axi4m2s;
        EastTXS2MxDO              : out t_axi4s2m;
        -- South
        SouthRXM2SxDO             : out t_axi4m2s;
        SouthRXS2MxDI             : in  t_axi4s2m;
        SouthTXM2SxDI             : in  t_axi4m2s;
        SouthTXS2MxDO             : out t_axi4s2m;
        -- West
        WestRXM2SxDO              : out t_axi4m2s;
        WestRXS2MxDI              : in  t_axi4s2m;
        WestTXM2SxDI              : in  t_axi4m2s;
        WestTXS2MxDO              : out t_axi4s2m;
        -- Axi4 Framing UFC Interface
        -- North
        NorthRXUFCM2SxDO          : out t_axi4ufcm2s_rx;
        NorthTXUFCM2SxDI          : in  t_axi4ufcm2s_tx;
        NorthTXUFCS2MxDO          : out t_axi4ufcs2m_tx;
        -- East
        EastRXUFCM2SxDO           : out t_axi4ufcm2s_rx;
        EastTXUFCM2SxDI           : in  t_axi4ufcm2s_tx;
        EastTXUFCS2MxDO           : out t_axi4ufcs2m_tx;
        -- South
        SouthRXUFCM2SxDO          : out t_axi4ufcm2s_rx;
        SouthTXUFCM2SxDI          : in  t_axi4ufcm2s_tx;
        SouthTXUFCS2MxDO          : out t_axi4ufcs2m_tx;
        -- West
        WestRXUFCM2SxDO           : out t_axi4ufcm2s_rx;
        WestTXUFCM2SxDI           : in  t_axi4ufcm2s_tx;
        WestTXUFCS2MxDO           : out t_axi4ufcs2m_tx;
        -- Axi4 Framing NFC Interface
        -- North
        NorthRXNFCM2SxDO          : out t_axi4nfcm2s;
        NorthTXNFCM2SxDI          : in  t_axi4nfcm2s;
        NorthTXNFCS2MxDO          : out t_axi4nfcs2m;
        -- East
        EastRXNFCM2SxDO           : out t_axi4nfcm2s;
        EastTXNFCM2SxDI           : in  t_axi4nfcm2s;
        EastTXNFCS2MxDO           : out t_axi4nfcs2m;
        -- South
        SouthRXNFCM2SxDO          : out t_axi4nfcm2s;
        SouthTXNFCM2SxDI          : in  t_axi4nfcm2s;
        SouthTXNFCS2mxDO          : out t_axi4nfcs2m;
        -- West
        WestRXNFCM2SxDO           : out t_axi4nfcm2s;
        WestTXNFCM2SxDI           : in  t_axi4nfcm2s;
        WestTXNFCS2MxDO           : out t_axi4nfcs2m;
        -- Aurora Ctrl + Status
        AuroraCtrlxDI             : in  t_aurora_control;
        AuroraStatusxDO           : out t_aurora_status;
        -- DRP Port
        AuroraDRPM2SxDI           : in  t_drpm2s_vector((C_NB_GTP_CORE - 1) downto 0);
        AuroraDRPS2MxDO           : out t_drps2m_vector((C_NB_GTP_CORE - 1) downto 0);
        -- RX Fifo Status
        -- North
        NorthRXFifoStatusxDO      : out t_axi4fifo_status;
        -- East
        EastRXFifoStatusxDO       : out t_axi4fifo_status;
        -- South
        SouthRXFifoStatusxDO      : out t_axi4fifo_status;
        -- West
        WestRXFifoStatusxDO       : out t_axi4fifo_status;
        -- Axis Fifo Error
        AxisFifoErrorxDO          : out t_axi4fifo_error);

end scalp_aurora_phy;


architecture arch of scalp_aurora_phy is

    signal GTRefClkxC               : t_gt_ref_slave_clk                            := C_GT_REF_NO_SLAVE_CLK;
    signal AuroraClkInxC            : t_aurora_slave_clk                            := C_AURORA_NO_SLAVE_CLK;
    signal AuroraClkOutxC           : t_aurora_master_clk                           := C_AURORA_NO_MASTER_CLK;
    signal AuroraResetInxR          : t_aurora_slave_reset                          := C_AURORA_NO_SLAVE_RESET;
    signal AuroraResetOutxR         : t_aurora_master_link_reset                    := C_AURORA_NO_MASTER_LINK_RESET;
    signal RXResetxR                : t_rx_reset                                    := C_NO_RX_RESET;
    signal RXFifoRstxRAN            : t_rx_fifo_reset                               := C_NO_RX_FIFO_RESET;
    signal RXFifoResetDonexD        : t_rx_fifo_reset_done                          := C_NO_RX_FIFO_RESET_DONE;
    signal RXFifoResetDoneDelayedxD : t_rx_fifo_reset_done                          := C_NO_RX_FIFO_RESET_DONE;
    signal GTPFromNorthxD           : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToNorthxD             : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromEastxD            : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToEastxD              : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromSouthxD           : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToSouthxD             : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal GTPFromWestxD            : t_aurora_gtp_diff_io_rx                       := C_AURORA_NO_GTP_DIFF_IO_RX;
    signal GTPToWestxD              : t_aurora_gtp_diff_io_tx                       := C_AURORA_NO_GTP_DIFF_IO_TX;
    signal NorthRXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal NorthTXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal NorthTXS2MxD             : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal EastRXM2SxD              : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal EastTXM2SxD              : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal EastTXS2MxD              : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal SouthRXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal SouthTXM2SxD             : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal SouthTXS2MxD             : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal WestRXM2SxD              : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal WestTXM2SxD              : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal WestTXS2MxD              : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal NorthRXUFCM2SxD          : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal NorthTXUFCM2SxD          : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal NorthTXUFCS2MxD          : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal EastRXUFCM2SxD           : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal EastTXUFCM2SxD           : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal EastTXUFCS2MxD           : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal SouthRXUFCM2SxD          : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal SouthTXUFCM2SxD          : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal SouthTXUFCS2MxD          : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal WestRXUFCM2SxD           : t_axi4ufcm2s_rx                               := C_NO_AXI4_UFC_M2S_RX;
    signal WestTXUFCM2SxD           : t_axi4ufcm2s_tx                               := C_NO_AXI4_UFC_M2S_TX;
    signal WestTXUFCS2MxD           : t_axi4ufcs2m_tx                               := C_NO_AXI4_UFC_S2M_TX;
    signal NorthRXNFCM2SxD          : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal NorthTXNFCM2SxD          : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal NorthTXNFCS2MxD          : t_axi4nfcs2m                                  := C_NO_AXI4_NFC_S2M;
    signal EastRXNFCM2SxD           : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal EastTXNFCM2SxD           : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal EastTXNFCS2MxD           : t_axi4nfcs2m                                  := C_NO_AXI4_NFC_S2M;
    signal SouthRXNFCM2SxD          : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal SouthTXNFCM2SxD          : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal SouthTXNFCS2mxD          : t_axi4nfcs2m                                  := C_NO_AXI4_NFC_S2M;
    signal WestRXNFCM2SxD           : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal WestTXNFCM2SxD           : t_axi4nfcm2s                                  := C_NO_AXI4_NFC_M2S;
    signal WestTXNFCS2MxD           : t_axi4nfcs2m                                  := C_NO_AXI4_NFC_S2M;
    signal AuroraCtrlxD             : t_aurora_control                              := C_AURORA_NO_CONTROL;
    signal AuroraStatusxD           : t_aurora_status                               := C_AURORA_NO_STATUS;
    signal AuroraDRPM2SxD           : t_drpm2s_vector((C_NB_GTP_CORE - 1) downto 0) := (others => C_NO_DRP_M2S);
    signal AuroraDRPS2MxD           : t_drps2m_vector((C_NB_GTP_CORE - 1) downto 0) := (others => C_NO_DRP_S2M);
    signal NorthRXFifoStatusxD      : t_axi4fifo_status                             := C_NO_AXI4_FIFO_STATUS;
    signal EastRXFifoStatusxD       : t_axi4fifo_status                             := C_NO_AXI4_FIFO_STATUS;
    signal SouthRXFifoStatusxD      : t_axi4fifo_status                             := C_NO_AXI4_FIFO_STATUS;
    signal WestRXFifoStatusxD       : t_axi4fifo_status                             := C_NO_AXI4_FIFO_STATUS;
    signal NorthFifoRXS2MxD         : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal NorthFifoTXM2SxD         : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal NorthFifoTXS2MxD         : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal EastFifoRXS2MxD          : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal EastFifoTXM2SxD          : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal EastFifoTXS2MxD          : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal SouthFifoRXS2MxD         : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal SouthFifoTXM2SxD         : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal SouthFifoTXS2MxD         : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal WestFifoRXS2MxD          : t_axi4s2m                                     := C_NO_AXI4_S2M;
    signal WestFifoTXM2SxD          : t_axi4m2s                                     := C_NO_AXI4_M2S;
    signal WestFifoTXS2MxD          : t_axi4s2m                                     := C_NO_AXI4_S2M;

    -- Attributes
    attribute mark_debug                             : string;
    attribute keep                                   : string;
    --
    attribute mark_debug of RXFifoResetDonexD        : signal is "true";
    attribute keep of RXFifoResetDonexD              : signal is "true";
    attribute mark_debug of RXFifoResetDoneDelayedxD : signal is "true";
    attribute keep of RXFifoResetDoneDelayedxD       : signal is "true";
    -- North
    attribute mark_debug of NorthTXM2SxD             : signal is "true";
    attribute keep of NorthTXM2SxD                   : signal is "true";
    attribute mark_debug of NorthTXS2MxD             : signal is "true";
    attribute keep of NorthTXS2MxD                   : signal is "true";
    attribute mark_debug of NorthRXM2SxD             : signal is "true";
    attribute keep of NorthRXM2SxD                   : signal is "true";
    attribute mark_debug of NorthTXNFCM2SxD          : signal is "true";
    attribute keep of NorthTXNFCM2SxD                : signal is "true";
    attribute mark_debug of NorthTXNFCS2MxD          : signal is "true";
    attribute keep of NorthTXNFCS2MxD                : signal is "true";
    attribute mark_debug of NorthRXNFCM2SxD          : signal is "true";
    attribute keep of NorthRXNFCM2SxD                : signal is "true";
    -- East
    -- attribute mark_debug of EastTXM2SxD              : signal is "true";
    -- attribute keep of EastTXM2SxD                    : signal is "true";
    -- attribute mark_debug of EastTXS2MxD              : signal is "true";
    -- attribute keep of EastTXS2MxD                    : signal is "true";
    -- attribute mark_debug of EastRXM2SxD              : signal is "true";
    -- attribute keep of EastRXM2SxD                    : signal is "true";
    -- attribute mark_debug of EastTXNFCM2SxD           : signal is "true";
    -- attribute keep of EastTXNFCM2SxD                 : signal is "true";
    -- attribute mark_debug of EastTXNFCS2MxD           : signal is "true";
    -- attribute keep of EastTXNFCS2MxD                 : signal is "true";
    -- attribute mark_debug of EastRXNFCM2SxD           : signal is "true";
    -- attribute keep of EastRXNFCM2SxD                 : signal is "true";
    -- South
    attribute mark_debug of SouthTXM2SxD             : signal is "true";
    attribute keep of SouthTXM2SxD                   : signal is "true";
    attribute mark_debug of SouthTXS2MxD             : signal is "true";
    attribute keep of SouthTXS2MxD                   : signal is "true";
    attribute mark_debug of SouthRXM2SxD             : signal is "true";
    attribute keep of SouthRXM2SxD                   : signal is "true";
    attribute mark_debug of SouthTXNFCM2SxD          : signal is "true";
    attribute keep of SouthTXNFCM2SxD                : signal is "true";
    attribute mark_debug of SouthTXNFCS2MxD          : signal is "true";
    attribute keep of SouthTXNFCS2MxD                : signal is "true";
    attribute mark_debug of SouthRXNFCM2SxD          : signal is "true";
    attribute keep of SouthRXNFCM2SxD                : signal is "true";
    -- West
    -- attribute mark_debug of WestTXM2SxD              : signal is "true";
    -- attribute keep of WestTXM2SxD                    : signal is "true";
    -- attribute mark_debug of WestTXS2MxD              : signal is "true";
    -- attribute keep of WestTXS2MxD                    : signal is "true";
    -- attribute mark_debug of WestRXM2SxD              : signal is "true";
    -- attribute keep of WestRXM2SxD                    : signal is "true";
    -- attribute mark_debug of WestTXNFCM2SxD           : signal is "true";
    -- attribute keep of WestTXNFCM2SxD                 : signal is "true";
    -- attribute mark_debug of WestTXNFCS2MxD           : signal is "true";
    -- attribute keep of WestTXNFCS2MxD                 : signal is "true";
    -- attribute mark_debug of WestRXNFCM2SxD           : signal is "true";
    -- attribute keep of WestRXNFCM2SxD                 : signal is "true";

begin

    EntityIOxB : block is
    begin  -- block EntityIOxB

        ClockxB : block is
        begin  -- block ClockxB

            GTRefClkxAS     : GTRefClkxC    <= GTRefClkxCI;
            AuroraClkInxAS  : AuroraClkInxC <= AuroraClkxCI;
            AuroraClkOutxAS : AuroraClkxCO  <= AuroraClkOutxC;

        end block ClockxB;

        ResetxB : block is
        begin  -- block ResetxB

            AuroraResetInxAS          : AuroraResetInxR           <= AuroraResetxRI;
            AuroraResetOutxAS         : AuroraResetxRO            <= AuroraResetOutxR;
            RXResetxAS                : RXResetxR                 <= RXResetxRI;
            RXFifoResetDonexAS        : RXFifoResetDonexDO        <= RXFifoResetDonexD;
            RXFifoResetDoneDelayedxAS : RXFifoResetDoneDelayedxDO <= RXFifoResetDoneDelayedxD;

        end block ResetxB;

        GTPxB : block is
        begin  -- block GTPxB

            -- North
            GTPFromNorthxAS : GTPFromNorthxD <= GTPFromNorthxDI;
            GTPToNorthxAS   : GTPToNorthxDO  <= GTPToNorthxD;
            -- East
            GTPFromEastxAS  : GTPFromEastxD  <= GTPFromEastxDI;
            GTPToEastxAS    : GTPToEastxDO   <= GTPToEastxD;
            -- South
            GTPFromSouthxAS : GTPFromSouthxD <= GTPFromSouthxDI;
            GTPToSouthxAS   : GTPToSouthxDO  <= GTPToSouthxD;
            -- West
            GTPFromWestxAS  : GTPFromWestxD  <= GTPFromWestxDI;
            GTPToWestxAS    : GTPToWestxDO   <= GTPToWestxD;

        end block GTPxB;

        TXxB : block is
        begin  -- block TXxB

            AXIxB : block is
            begin  -- block AXIxB

                -- North
                NorthTXM2SxAS : NorthTXM2SxD  <= NorthTXM2SxDI;
                NorthTXS2MxAS : NorthTXS2MxDO <= NorthTXS2MxD;
                -- East
                EastTXM2SxAS  : EastTXM2SxD   <= EastTXM2SxDI;
                EastTXS2MxAS  : EastTXS2MxDO  <= EastTXS2MxD;
                -- South
                SouthTXM2SxAS : SouthTXM2SxD  <= SouthTXM2SxDI;
                SouthTXS2MxAS : SouthTXS2MxDO <= SouthTXS2MxD;
                -- West
                WestTXM2SxAS  : WestTXM2SxD   <= WestTXM2SxDI;
                WestTXS2MxAS  : WestTXS2MxDO  <= WestTXS2MxD;

            end block AXIxB;

            UFCxB : block is
            begin  -- block UFCxB

                -- North
                NorthTXUFCM2SxAS : NorthTXUFCM2SxD  <= NorthTXUFCM2SxDI;
                NorthTXUFCS2MxAS : NorthTXUFCS2MxDO <= NorthTXUFCS2MxD;
                -- East
                EastTXUFCM2SxAS  : EastTXUFCM2SxD   <= EastTXUFCM2SxDI;
                EastTXUFCS2MxAS  : EastTXUFCS2MxDO  <= EastTXUFCS2MxD;
                -- South
                SouthTXUFCM2SxAS : SouthTXUFCM2SxD  <= SouthTXUFCM2SxDI;
                SouthTXUFCS2MxAS : SouthTXUFCS2MxDO <= SouthTXUFCS2MxD;
                -- West
                WestTXUFCM2SxAS  : WestTXUFCM2SxD   <= WestTXUFCM2SxDI;
                WestTXUFCS2MxAS  : WestTXUFCS2MxDO  <= WestTXUFCS2MxD;

            end block UFCxB;

        end block TXxB;

        RXxB : block is
        begin  -- block RXxB

            UFCxB : block is
            begin  -- block UFCxB

                -- North
                NorthRXUFCM2SxAS : NorthRXUFCM2SxDO <= NorthRXUFCM2SxD;
                -- East
                EastRXUFCM2SxAS  : EastRXUFCM2SxDO  <= EastRXUFCM2SxD;
                -- East
                SouthRXUFCM2SxAS : SouthRXUFCM2SxDO <= SouthRXUFCM2SxD;
                -- West
                WestRXUFCM2SxAS  : WestRXUFCM2SxDO  <= WestRXUFCM2SxD;

            end block UFCxB;

        end block RXxB;

        CtrlStatusxB : block is
        begin  -- block CtrlStatusxB

            AuroraCtrlxAS   : AuroraCtrlxD    <= AuroraCtrlxDI;
            AuroraStatusxAS : AuroraStatusxDO <= AuroraStatusxD;

        end block CtrlStatusxB;

        DRPxB : block is
        begin  -- block DRPxB

            AuroraDRPM2SxAS : AuroraDRPM2SxD  <= AuroraDRPM2SxDI;
            AuroraDRPS2MxAS : AuroraDRPS2MxDO <= AuroraDRPS2MxD;

        end block DRPxB;

    end block EntityIOxB;

    ScalpAuroraPhyxB : block is
    begin  -- block ScalpAuroraPhyxB

        ScalpAuroraPhyWrapperxI : entity work.scalp_aurora_phy_wrapper
            generic map (
                C_DEBUG_MODE => C_DEBUG_MODE)
            port map (
                GTRefClkxCI      => GTRefClkxC,
                AuroraClkxCI     => AuroraClkInxC,
                AuroraClkxCO     => AuroraClkOutxC,
                AuroraResetxRI   => AuroraResetInxR,
                AuroraResetxRO   => AuroraResetOutxR,
                GTPFromNorthxDI  => GTPFromNorthxD,
                GTPToNorthxDO    => GTPToNorthxD,
                GTPFromEastxDI   => GTPFromEastxD,
                GTPToEastxDO     => GTPToEastxD,
                GTPFromSouthxDI  => GTPFromSouthxD,
                GTPToSouthxDO    => GTPToSouthxD,
                GTPFromWestxDI   => GTPFromWestxD,
                GTPToWestxDO     => GTPToWestxD,
                NorthRXM2SxDO    => NorthRXM2SxD,
                NorthTXM2SxDI    => NorthTXM2SxD,
                NorthTXS2MxDO    => NorthTXS2MxD,
                EastRXM2SxDO     => EastRXM2SxD,
                EastTXM2SxDI     => EastTXM2SxD,
                EastTXS2MxDO     => EastTXS2MxD,
                SouthRXM2SxDO    => SouthRXM2SxD,
                SouthTXM2SxDI    => SouthTXM2SxD,
                SouthTXS2MxDO    => SouthTXS2MxD,
                WestRXM2SxDO     => WestRXM2SxD,
                WestTXM2SxDI     => WestTXM2SxD,
                WestTXS2MxDO     => WestTXS2MxD,
                NorthRXUFCM2SxDO => NorthRXUFCM2SxD,
                NorthTXUFCM2SxDI => NorthTXUFCM2SxD,
                NorthTXUFCS2MxDO => NorthTXUFCS2MxD,
                EastRXUFCM2SxDO  => EastRXUFCM2SxD,
                EastTXUFCM2SxDI  => EastTXUFCM2SxD,
                EastTXUFCS2MxDO  => EastTXUFCS2MxD,
                SouthRXUFCM2SxDO => SouthRXUFCM2SxD,
                SouthTXUFCM2SxDI => SouthTXUFCM2SxD,
                SouthTXUFCS2MxDO => SouthTXUFCS2MxD,
                WestRXUFCM2SxDO  => WestRXUFCM2SxD,
                WestTXUFCM2SxDI  => WestTXUFCM2SxD,
                WestTXUFCS2MxDO  => WestTXUFCS2MxD,
                NorthRXNFCM2SxDO => NorthRXNFCM2SxD,
                NorthTXNFCM2SxDI => NorthTXNFCM2SxD,
                NorthTXNFCS2MxDO => NorthTXNFCS2MxD,
                EastRXNFCM2SxDO  => EastRXNFCM2SxD,
                EastTXNFCM2SxDI  => EastTXNFCM2SxD,
                EastTXNFCS2MxDO  => EastTXNFCS2MxD,
                SouthRXNFCM2SxDO => SouthRXNFCM2SxD,
                SouthTXNFCM2SxDI => SouthTXNFCM2SxD,
                SouthTXNFCS2mxDO => SouthTXNFCS2mxD,
                WestRXNFCM2SxDO  => WestRXNFCM2SxD,
                WestTXNFCM2SxDI  => WestTXNFCM2SxD,
                WestTXNFCS2MxDO  => WestTXNFCS2MxD,
                AuroraCtrlxDI    => AuroraCtrlxD,
                AuroraStatusxDO  => AuroraStatusxD,
                AuroraDRPM2SxDI  => AuroraDRPM2SxD,
                AuroraDRPS2MxDO  => AuroraDRPS2MxD);

    end block ScalpAuroraPhyxB;

    RXModexB : block is
    begin  -- block RXModexB

        RXFifoModexG : if C_RX_FIFO_MODE = false generate

            EntityIOxB : block is
            begin  -- block EntityIOxB

                TXxB : block is
                begin  -- block TXxB

                    NFCxB : block is
                    begin  -- block NFCxB

                        -- North
                        NorthTXNFCM2SxAS : NorthTXNFCM2SxD  <= NorthTXNFCM2SxDI;
                        NorthTXNFCS2MxAS : NorthTXNFCS2MxDO <= NorthTXNFCS2MxD;
                        -- East
                        EastTXNFCM2SxAS  : EastTXNFCM2SxD   <= EastTXNFCM2SxDI;
                        EastTXNFCS2MxAS  : EastTXNFCS2MxDO  <= EastTXNFCS2MxD;
                        -- South
                        SouthTXNFCM2SxAS : SouthTXNFCM2SxD  <= SouthTXNFCM2SxDI;
                        SouthTXNFCS2MxAS : SouthTXNFCS2MxDO <= SouthTXNFCS2MxD;
                        -- West
                        WestTXNFCM2SxAS  : WestTXNFCM2SxD   <= WestTXNFCM2SxDI;
                        WestTXNFCS2MxAS  : WestTXNFCS2MxDO  <= WestTXNFCS2MxD;

                    end block NFCxB;

                end block TXxB;

                RXxB : block is
                begin  -- block RXxB

                    AXIxB : block is
                    begin  -- block AXIxB

                        -- North
                        NorthRXM2SxAS : NorthRXM2SxDO <= NorthRXM2SxD;
                        -- East
                        EastRXM2SxAS  : EastRXM2SxDO  <= EastRXM2SxD;
                        -- South
                        SouthRXM2SxAS : SouthRXM2SxDO <= SouthRXM2SxD;
                        -- West
                        WestRXM2SxAS  : WestRXM2SxDO  <= WestRXM2SxD;

                    end block AXIxB;

                    NFCxB : block is
                    begin  -- block NFCxB

                        -- North
                        NorthRXNFCM2SxAS : NorthRXNFCM2SxDO <= NorthRXNFCM2SxD;
                        -- East
                        EastRXNFCM2SxAS  : EastRXNFCM2SxDO  <= EastRXNFCM2SxD;
                        -- South
                        SouthRXNFCM2SxAS : SouthRXNFCM2SxDO <= SouthRXNFCM2SxD;
                        -- West
                        WestRXNFCM2SxAS  : WestRXNFCM2SxDO  <= WestRXNFCM2SxD;

                    end block NFCxB;

                end block RXxB;

                RXFifoStatusxB : block is
                begin  -- block RXFifoStatusxB

                    NorthRXFifoStatusxAS : NorthRXFifoStatusxDO <= C_NO_AXI4_FIFO_STATUS;
                    EastRXFifoStatusxAS  : EastRXFifoStatusxDO  <= C_NO_AXI4_FIFO_STATUS;
                    SouthRXFifoStatusxAS : SouthRXFifoStatusxDO <= C_NO_AXI4_FIFO_STATUS;
                    WestRXFifoStatusxAS  : WestRXFifoStatusxDO  <= C_NO_AXI4_FIFO_STATUS;
                    AxisFifoErrorxAS     : AxisFifoErrorxDO     <= C_NO_AXI4_FIFO_ERROR;

                end block RXFifoStatusxB;

            end block EntityIOxB;

        elsif C_RX_FIFO_MODE = true generate

            EntityIOxB : block is
            begin  -- block EntityIOxB

                -- North
                NorthRXNFCM2SxAS : NorthRXNFCM2SxDO <= C_NO_AXI4_NFC_M2S;
                NorthTXNFCS2MxAS : NorthTXNFCS2MxDO <= C_NO_AXI4_NFC_S2M;
                -- East
                EastRXNFCM2SxAS  : EastRXNFCM2SxDO  <= C_NO_AXI4_NFC_M2S;
                EastTXNFCS2MxAS  : EastTXNFCS2MxDO  <= C_NO_AXI4_NFC_S2M;
                -- South
                SouthRXNFCM2SxAS : SouthRXNFCM2SxDO <= C_NO_AXI4_NFC_M2S;
                SouthTXNFCS2MxAS : SouthTXNFCS2MxDO <= C_NO_AXI4_NFC_S2M;
                -- West
                WestRXNFCM2SxAS  : WestRXNFCM2SxDO  <= C_NO_AXI4_NFC_M2S;
                WestTXNFCS2MxAS  : WestTXNFCS2MxDO  <= C_NO_AXI4_NFC_S2M;

            end block EntityIOxB;

            RXFifoxB : block is
            begin  -- block RXFifoxB

                NorthRXFifoRstxAS : RXFifoRstxRAN.NorthxR <= (not AuroraClkOutxC.PllNotLockedxS) and
                                                             (not RXResetxR.FifoResetxR.NorthxR);
                EastRXFifoRstxAS : RXFifoRstxRAN.EastxR <= (not AuroraClkOutxC.PllNotLockedxS) and
                                                           (not RXResetxR.FifoResetxR.EastxR);
                SouthRXFifoRstxAS : RXFifoRstxRAN.SouthxR <= (not AuroraClkOutxC.PllNotLockedxS) and
                                                             (not RXResetxR.FifoResetxR.SouthxR);
                WestRXFifoRstxAS : RXFifoRstxRAN.WestxR <= (not AuroraClkOutxC.PllNotLockedxS) and
                                                           (not RXResetxR.FifoResetxR.WestxR);
                -- North
                NorthRXM2SxAS        : NorthRXM2SxDO        <= NorthFifoTXM2SxD;
                NorthRXS2MxAS        : NorthFifoTXS2MxD     <= NorthRXS2MxDI;
                NorthRXFifoStatusxAS : NorthRXFifoStatusxDO <= NorthRXFifoStatusxD;
                -- East
                EastRXM2SxAS         : EastRXM2SxDO         <= EastFifoTXM2SxD;
                EastRXS2MxAS         : EastFifoTXS2MxD      <= EastRXS2MxDI;
                EastRXFifoStatusxAS  : EastRXFifoStatusxDO  <= EastRXFifoStatusxD;
                -- South
                SouthRXM2SxAS        : SouthRXM2SxDO        <= SouthFifoTXM2SxD;
                SouthRXS2MxAS        : SouthFifoTXS2MxD     <= SouthRXS2MxDI;
                SouthRXFifoStatusxAS : SouthRXFifoStatusxDO <= SouthRXFifoStatusxD;
                -- West
                WestRXM2SxAS         : WestRXM2SxDO         <= WestFifoTXM2SxD;
                WestRXS2MxAS         : WestFifoTXS2MxD      <= WestRXS2MxDI;
                WestRXFifoStatusxAS  : WestRXFifoStatusxDO  <= WestRXFifoStatusxD;

                ScalpAuroraPhyRxFifoxI : entity work.scalp_aurora_phy_rx_fifo
                    port map (
                        -- Clocks and Reset
                        RXClkxCI           => AuroraClkOutxC.UserClkxC,
                        TXClkxCI           => AuroraClkOutxC.UserClkxC,
                        RXRstxRANI         => RXFifoRstxRAN,
                        -- North
                        -- Fifo RX Side
                        NorthRXM2SxDI      => NorthRXM2SxD,
                        NorthRXS2MxDO      => NorthFifoRXS2MxD,
                        -- Fifo TX Side
                        NorthTXM2SxDO      => NorthFifoTXM2SxD,
                        NorthTXS2MxDI      => NorthFifoTXS2MxD,
                        NorthFifoStatusxDO => NorthRXFifoStatusxD,
                        -- East
                        -- Fifo RX Side
                        EastRXM2SxDI       => EastRXM2SxD,
                        EastRXS2MxDO       => EastFifoRXS2MxD,
                        -- Fifo TX Side
                        EastTXM2SxDO       => EastFifoTXM2SxD,
                        EastTXS2MxDI       => EastFifoTXS2MxD,
                        EastFifoStatusxDO  => EastRXFifoStatusxD,
                        -- South
                        -- Fifo RX Side
                        SouthRXM2SxDI      => SouthRXM2SxD,
                        SouthRXS2MxDO      => SouthFifoRXS2MxD,
                        -- Fifo TX Side
                        SouthTXM2SxDO      => SouthFifoTXM2SxD,
                        SouthTXS2MxDI      => SouthFifoTXS2MxD,
                        SouthFifoStatusxDO => SouthRXFifoStatusxD,
                        -- West
                        -- Fifo RX Side
                        WestRXM2SxDI       => WestRXM2SxD,
                        WestRXS2MxDO       => WestFifoRXS2MxD,
                        -- Fifo TX Side
                        WestTXM2SxDO       => WestFifoTXM2SxD,
                        WestTXS2MxDI       => WestFifoTXS2MxD,
                        WestFifoStatusxDO  => WestRXFifoStatusxD);

            end block RXFifoxB;

            RXFifoResetDonexB : block is

                signal NorthRXFifoResetDoneStatexDN : t_rx_fifo_reset_done_states := E_RESET;
                signal NorthRXFifoResetDoneStatexDP : t_rx_fifo_reset_done_states := E_RESET;
                signal EastRXFifoResetDoneStatexDN  : t_rx_fifo_reset_done_states := E_RESET;
                signal EastRXFifoResetDoneStatexDP  : t_rx_fifo_reset_done_states := E_RESET;
                signal SouthRXFifoResetDoneStatexDN : t_rx_fifo_reset_done_states := E_RESET;
                signal SouthRXFifoResetDoneStatexDP : t_rx_fifo_reset_done_states := E_RESET;
                signal WestRXFifoResetDoneStatexDN  : t_rx_fifo_reset_done_states := E_RESET;
                signal WestRXFifoResetDoneStatexDP  : t_rx_fifo_reset_done_states := E_RESET;
                signal AuroraPllLockedxS            : std_ulogic                  := '0';

                -- Attributes
                attribute mark_debug                                 : string;
                attribute keep                                       : string;
                --
                attribute mark_debug of NorthRXFifoResetDoneStatexDP : signal is "true";
                attribute keep of NorthRXFifoResetDoneStatexDP       : signal is "true";
                -- attribute mark_debug of EastRXFifoResetDoneStatexDP  : signal is "true";
                -- attribute keep of EastRXFifoResetDoneStatexDP        : signal is "true";
                attribute mark_debug of SouthRXFifoResetDoneStatexDP : signal is "true";
                attribute keep of SouthRXFifoResetDoneStatexDP       : signal is "true";
                -- attribute mark_debug of WestRXFifoResetDoneStatexDP  : signal is "true";
                -- attribute keep of WestRXFifoResetDoneStatexDP        : signal is "true";

            begin  -- block RXFifoResetDonexB

                AuroraPllLockedxAS : AuroraPllLockedxS <= not AuroraClkOutxC.PllNotLockedxS;

                NorthRXFifoResetDoneDelayxI : entity work.reset_delay_gen
                    generic map (
                        C_TICKS => C_RX_FIFO_RST_DONE_DLY_TICKS)
                    port map (
                        ClkxCI          => AuroraClkOutxC.UserClkxC,
                        PllLockedxSI    => AuroraPllLockedxS,
                        ResetxRI        => RXFifoResetDonexD.NorthxS,
                        ResetDelayedxRO => RXFifoResetDoneDelayedxD.NorthxS);

                EastRXFifoResetDoneDelayxI : entity work.reset_delay_gen
                    generic map (
                        C_TICKS => C_RX_FIFO_RST_DONE_DLY_TICKS)
                    port map (
                        ClkxCI          => AuroraClkOutxC.UserClkxC,
                        PllLockedxSI    => AuroraPllLockedxS,
                        ResetxRI        => RXFifoResetDonexD.EastxS,
                        ResetDelayedxRO => RXFifoResetDoneDelayedxD.EastxS);

                SouthRXFifoResetDoneDelayxI : entity work.reset_delay_gen
                    generic map (
                        C_TICKS => C_RX_FIFO_RST_DONE_DLY_TICKS)
                    port map (
                        ClkxCI          => AuroraClkOutxC.UserClkxC,
                        PllLockedxSI    => AuroraPllLockedxS,
                        ResetxRI        => RXFifoResetDonexD.SouthxS,
                        ResetDelayedxRO => RXFifoResetDoneDelayedxD.SouthxS);

                WestRXFifoResetDoneDelayxI : entity work.reset_delay_gen
                    generic map (
                        C_TICKS => C_RX_FIFO_RST_DONE_DLY_TICKS)
                    port map (
                        ClkxCI          => AuroraClkOutxC.UserClkxC,
                        PllLockedxSI    => AuroraPllLockedxS,
                        ResetxRI        => RXFifoResetDonexD.WestxS,
                        ResetDelayedxRO => RXFifoResetDoneDelayedxD.WestxS);

                NorthUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                            RXFifoRstxRAN.NorthxR) is
                begin  -- process NorthUpdateRegxP
                    if RXFifoRstxRAN.NorthxR = '0' then
                        NorthRXFifoResetDoneStatexDP <= E_RESET;
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        NorthRXFifoResetDoneStatexDP <= NorthRXFifoResetDoneStatexDN;
                    end if;
                end process NorthUpdateRegxP;

                EastUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                           RXFifoRstxRAN.EastxR) is
                begin  -- process EastUpdateRegxP
                    if RXFifoRstxRAN.EastxR = '0' then
                        EastRXFifoResetDoneStatexDP <= E_RESET;
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        EastRXFifoResetDoneStatexDP <= EastRXFifoResetDoneStatexDN;
                    end if;

                end process EastUpdateRegxP;

                SouthUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                            RXFifoRstxRAN.SouthxR) is
                begin  -- process SouthUpdateRegxP
                    if RXFifoRstxRAN.SouthxR = '0' then
                        SouthRXFifoResetDoneStatexDP <= E_RESET;
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        SouthRXFifoResetDoneStatexDP <= SouthRXFifoResetDoneStatexDN;
                    end if;

                end process SouthUpdateRegxP;

                WestUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                           RXFifoRstxRAN.WestxR) is
                begin  -- process WestUpdateRegxP
                    if RXFifoRstxRAN.WestxR = '0' then
                        WestRXFifoResetDoneStatexDP <= E_RESET;
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        WestRXFifoResetDoneStatexDP <= WestRXFifoResetDoneStatexDN;
                    end if;

                end process WestUpdateRegxP;

                NorthRXFifoResetDonexP : process (NorthRXFifoResetDoneStatexDP,
                                                  NorthRXFifoStatusxD.ProgEmptyxS,
                                                  NorthRXFifoStatusxD.ProgFullxS) is
                begin  -- process NorthRXFifoResetDonexP
                    -- Default values
                    NorthRXFifoResetDoneStatexDN <= NorthRXFifoResetDoneStatexDP;
                    RXFifoResetDonexD.NorthxS    <= '0';

                    case NorthRXFifoResetDoneStatexDP is
                        when E_RESET =>
                                NorthRXFifoResetDoneStatexDN <= E_WAIT_DONE;

                        when E_WAIT_DONE =>
                                if (NorthRXFifoStatusxD.ProgFullxS = '1') and
                                    (NorthRXFifoStatusxD.ProgEmptyxS = '1') then
                                    NorthRXFifoResetDoneStatexDN <= E_WAIT_DONE;
                                else
                                    RXFifoResetDonexD.NorthxS    <= '1';
                                    NorthRXFifoResetDoneStatexDN <= E_IS_DONE;
                                end if;

                        when E_IS_DONE =>
                                RXFifoResetDonexD.NorthxS <= '1';

                        when others => null;
                    end case;
                end process NorthRXFifoResetDonexP;

                EastRXFifoResetDonexP : process (EastRXFifoResetDoneStatexDP,
                                                 EastRXFifoStatusxD.ProgEmptyxS,
                                                 EastRXFifoStatusxD.ProgFullxS) is
                begin  -- process EastRXFifoResetDonexP
                    -- Default values
                    EastRXFifoResetDoneStatexDN <= EastRXFifoResetDoneStatexDP;
                    RXFifoResetDonexD.EastxS    <= '0';

                    case EastRXFifoResetDoneStatexDP is
                        when E_RESET =>
                                EastRXFifoResetDoneStatexDN <= E_WAIT_DONE;

                        when E_WAIT_DONE =>
                                if (EastRXFifoStatusxD.ProgFullxS = '1') and
                                    (EastRXFifoStatusxD.ProgEmptyxS = '1') then
                                    EastRXFifoResetDoneStatexDN <= E_WAIT_DONE;
                                else
                                    RXFifoResetDonexD.EastxS    <= '1';
                                    EastRXFifoResetDoneStatexDN <= E_IS_DONE;
                                end if;

                        when E_IS_DONE =>
                                RXFifoResetDonexD.EastxS <= '1';

                        when others => null;
                    end case;
                end process EastRXFifoResetDonexP;

                SouthRXFifoResetDonexP : process (SouthRXFifoResetDoneStatexDP,
                                                  SouthRXFifoStatusxD.ProgEmptyxS,
                                                  SouthRXFifoStatusxD.ProgFullxS) is
                begin  -- process SouthRXFifoResetDonexP
                    -- Default values
                    SouthRXFifoResetDoneStatexDN <= SouthRXFifoResetDoneStatexDP;
                    RXFifoResetDonexD.SouthxS    <= '0';

                    case SouthRXFifoResetDoneStatexDP is
                        when E_RESET =>
                                SouthRXFifoResetDoneStatexDN <= E_WAIT_DONE;

                        when E_WAIT_DONE =>
                                if (SouthRXFifoStatusxD.ProgFullxS = '1') and
                                    (SouthRXFifoStatusxD.ProgEmptyxS = '1') then
                                    SouthRXFifoResetDoneStatexDN <= E_WAIT_DONE;
                                else
                                    RXFifoResetDonexD.SouthxS    <= '1';
                                    SouthRXFifoResetDoneStatexDN <= E_IS_DONE;
                                end if;

                        when E_IS_DONE =>
                                RXFifoResetDonexD.SouthxS <= '1';

                        when others => null;
                    end case;
                end process SouthRXFifoResetDonexP;

                WestRXFifoResetDonexP : process (WestRXFifoResetDoneStatexDP,
                                                 WestRXFifoStatusxD.ProgEmptyxS,
                                                 WestRXFifoStatusxD.ProgFullxS) is
                begin  -- process WestRXFifoResetDonexP
                    -- Default values
                    WestRXFifoResetDoneStatexDN <= WestRXFifoResetDoneStatexDP;
                    RXFifoResetDonexD.WestxS    <= '0';

                    case WestRXFifoResetDoneStatexDP is
                        when E_RESET =>
                                WestRXFifoResetDoneStatexDN <= E_WAIT_DONE;

                        when E_WAIT_DONE =>
                                if (WestRXFifoStatusxD.ProgFullxS = '1') and
                                    (WestRXFifoStatusxD.ProgEmptyxS = '1') then
                                    WestRXFifoResetDoneStatexDN <= E_WAIT_DONE;
                                else
                                    RXFifoResetDonexD.WestxS    <= '1';
                                    WestRXFifoResetDoneStatexDN <= E_IS_DONE;
                                end if;

                        when E_IS_DONE =>
                                RXFifoResetDonexD.WestxS <= '1';

                        when others => null;
                    end case;
                end process WestRXFifoResetDonexP;

            end block RXFifoResetDonexB;

            RXBackPressurexB : block is

                signal BackPressureRstxR : t_rx_back_pressure_reset                               := C_NO_RX_BACK_PRESSURE_RESET;
                -- North
                signal NorthProgFullxSN  : std_ulogic                                             := '0';
                signal NorthProgFullxSP  : std_ulogic                                             := '0';
                signal NorthProgEmptyxSN : std_ulogic                                             := '0';
                signal NorthProgEmptyxSP : std_ulogic                                             := '0';
                signal NorthNFCStatexDN  : t_nfc_states                                           := C_NFC_IDLE;
                signal NorthNFCStatexDP  : t_nfc_states                                           := C_NFC_IDLE;
                signal NorthNFCDataxDN   : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal NorthNFCDataxDP   : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal NorthNFCValidxSN  : std_ulogic                                             := '0';
                signal NorthNFCValidxSP  : std_ulogic                                             := '0';
                -- East
                signal EastProgFullxSN   : std_ulogic                                             := '0';
                signal EastProgFullxSP   : std_ulogic                                             := '0';
                signal EastProgEmptyxSN  : std_ulogic                                             := '0';
                signal EastProgEmptyxSP  : std_ulogic                                             := '0';
                signal EastNFCStatexDN   : t_nfc_states                                           := C_NFC_IDLE;
                signal EastNFCStatexDP   : t_nfc_states                                           := C_NFC_IDLE;
                signal EastNFCDataxDN    : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal EastNFCDataxDP    : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal EastNFCValidxSN   : std_ulogic                                             := '0';
                signal EastNFCValidxSP   : std_ulogic                                             := '0';
                -- South
                signal SouthProgFullxSN  : std_ulogic                                             := '0';
                signal SouthProgFullxSP  : std_ulogic                                             := '0';
                signal SouthProgEmptyxSN : std_ulogic                                             := '0';
                signal SouthProgEmptyxSP : std_ulogic                                             := '0';
                signal SouthNFCStatexDN  : t_nfc_states                                           := C_NFC_IDLE;
                signal SouthNFCStatexDP  : t_nfc_states                                           := C_NFC_IDLE;
                signal SouthNFCDataxDN   : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal SouthNFCDataxDP   : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal SouthNFCValidxSN  : std_ulogic                                             := '0';
                signal SouthNFCValidxSP  : std_ulogic                                             := '0';
                -- West
                signal WestProgFullxSN   : std_ulogic                                             := '0';
                signal WestProgFullxSP   : std_ulogic                                             := '0';
                signal WestProgEmptyxSN  : std_ulogic                                             := '0';
                signal WestProgEmptyxSP  : std_ulogic                                             := '0';
                signal WestNFCStatexDN   : t_nfc_states                                           := C_NFC_IDLE;
                signal WestNFCStatexDP   : t_nfc_states                                           := C_NFC_IDLE;
                signal WestNFCDataxDN    : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal WestNFCDataxDP    : std_ulogic_vector((C_AXI4_NFC_DATA_SIZE - 1) downto 0) := (others => '0');
                signal WestNFCValidxSN   : std_ulogic                                             := '0';
                signal WestNFCValidxSP   : std_ulogic                                             := '0';
                -- Fifo Error
                signal AxisFifoErrorxDN  : t_axi4fifo_error                                       := C_NO_AXI4_FIFO_ERROR;
                signal AxisFifoErrorxDP  : t_axi4fifo_error                                       := C_NO_AXI4_FIFO_ERROR;

                -- Attributes
                attribute mark_debug                      : string;
                attribute keep                            : string;
                -- North
                attribute mark_debug of NorthNFCStatexDP  : signal is "true";
                attribute keep of NorthNFCStatexDP        : signal is "true";
                attribute mark_debug of NorthProgFullxSP  : signal is "true";
                attribute keep of NorthProgFullxSP        : signal is "true";
                attribute mark_debug of NorthProgEmptyxSP : signal is "true";
                attribute keep of NorthProgEmptyxSP       : signal is "true";
                -- East
                -- attribute mark_debug of EastNFCStatexDP   : signal is "true";
                -- attribute keep of EastNFCStatexDP         : signal is "true";
                -- attribute mark_debug of EastProgFullxSP   : signal is "true";
                -- attribute keep of EastProgFullxSP         : signal is "true";
                -- attribute mark_debug of EastProgEmptyxSP  : signal is "true";
                -- attribute keep of EastProgEmptyxSP        : signal is "true";
                -- South
                attribute mark_debug of SouthNFCStatexDP  : signal is "true";
                attribute keep of SouthNFCStatexDP        : signal is "true";
                attribute mark_debug of SouthProgFullxSP  : signal is "true";
                attribute keep of SouthProgFullxSP        : signal is "true";
                attribute mark_debug of SouthProgEmptyxSP : signal is "true";
                attribute keep of SouthProgEmptyxSP       : signal is "true";
                -- West
                -- attribute mark_debug of WestNFCStatexDP   : signal is "true";
                -- attribute keep of WestNFCStatexDP         : signal is "true";
                -- attribute mark_debug of WestProgFullxSP   : signal is "true";
                -- attribute keep of WestProgFullxSP         : signal is "true";
                -- attribute mark_debug of WestProgEmptyxSP  : signal is "true";
                -- attribute keep of WestProgEmptyxSP        : signal is "true";

            begin  -- block RXBackPressurexB

                NorthBackPressureRstxAS : BackPressureRstxR.NorthxR <= AuroraClkOutxC.PllNotLockedxS or
                                                                       RXResetxR.BackPressureResetxR.NorthxR;
                EastBackPressureRstxAS : BackPressureRstxR.EastxR <= AuroraClkOutxC.PllNotLockedxS or
                                                                     RXResetxR.BackPressureResetxR.EastxR;
                SouthBackPressureRstxAS : BackPressureRstxR.SouthxR <= AuroraClkOutxC.PllNotLockedxS or
                                                                       RXResetxR.BackPressureResetxR.SouthxR;
                WestBackPressureRstxAS : BackPressureRstxR.WestxR <= AuroraClkOutxC.PllNotLockedxS or
                                                                     RXResetxR.BackPressureResetxR.WestxR;
                -- NFC Data and valid signals
                NorthNFCDataxAS   : NorthTXNFCM2SxD.DataxD  <= NorthNFCDataxDP;
                NorthNFCValidxAS  : NorthTXNFCM2SxD.ValidxS <= NorthNFCValidxSP;
                NorthProgFullxAS  : NorthProgFullxSN        <= NorthRXFifoStatusxD.ProgFullxS;
                NorthProgEmptyxAS : NorthProgEmptyxSN       <= NorthRXFifoStatusxD.ProgEmptyxS;
                EastNFCDataxAS    : EastTXNFCM2SxD.DataxD   <= EastNFCDataxDP;
                EastNFCValidxAS   : EastTXNFCM2SxD.ValidxS  <= EastNFCValidxSP;
                EastProgFullxAS   : EastProgFullxSN         <= EastRXFifoStatusxD.ProgFullxS;
                EastProgEmptyxAS  : EastProgEmptyxSN        <= EastRXFifoStatusxD.ProgEmptyxS;
                SouthNFCDataxAS   : SouthTXNFCM2SxD.DataxD  <= SouthNFCDataxDP;
                SouthNFCValidxAS  : SouthTXNFCM2SxD.ValidxS <= SouthNFCValidxSP;
                SouthProgFullxAS  : SouthProgFullxSN        <= SouthRXFifoStatusxD.ProgFullxS;
                SouthProgEmptyxAS : SouthProgEmptyxSN       <= SouthRXFifoStatusxD.ProgEmptyxS;
                WestNFCDataxAS    : WestTXNFCM2SxD.DataxD   <= WestNFCDataxDP;
                WestNFCValidxAS   : WestTXNFCM2SxD.ValidxS  <= WestNFCValidxSP;
                WestProgFullxAS   : WestProgFullxSN         <= WestRXFifoStatusxD.ProgFullxS;
                WestProgEmptyxAS  : WestProgEmptyxSN        <= WestRXFifoStatusxD.ProgEmptyxS;
                Axi4FifoErrorxAS  : AxisFifoErrorxDO        <= AxisFifoErrorxDP;

                NorthUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                            BackPressureRstxR.NorthxR) is
                begin  -- process NorthUpdateRegxP
                    if BackPressureRstxR.NorthxR = '1' then
                        NorthProgFullxSP                <= '0';
                        NorthProgEmptyxSP               <= '0';
                        NorthNFCStatexDP                <= C_NFC_IDLE;
                        NorthNFCDataxDP                 <= (others => '0');
                        NorthNFCValidxSP                <= '0';
                        AxisFifoErrorxDP.NorthCounterxD <= (others => '0');
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        NorthProgFullxSP                <= NorthProgFullxSN;
                        NorthProgEmptyxSP               <= NorthProgEmptyxSN;
                        NorthNFCStatexDP                <= NorthNFCStatexDN;
                        NorthNFCDataxDP                 <= NorthNFCDataxDN;
                        NorthNFCValidxSP                <= NorthNFCValidxSN;
                        AxisFifoErrorxDP.NorthCounterxD <= AxisFifoErrorxDN.NorthCounterxD;
                    end if;
                end process NorthUpdateRegxP;

                EastUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                           BackPressureRstxR.EastxR) is
                begin  -- process EastUpdateRegxP
                    if BackPressureRstxR.EastxR = '1' then
                        EastProgFullxSP                <= '0';
                        EastProgEmptyxSP               <= '0';
                        EastNFCStatexDP                <= C_NFC_IDLE;
                        EastNFCDataxDP                 <= (others => '0');
                        EastNFCValidxSP                <= '0';
                        AxisFifoErrorxDP.EastCounterxD <= (others => '0');
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        EastProgFullxSP                <= EastProgFullxSN;
                        EastProgEmptyxSP               <= EastProgEmptyxSN;
                        EastNFCStatexDP                <= EastNFCStatexDN;
                        EastNFCDataxDP                 <= EastNFCDataxDN;
                        EastNFCValidxSP                <= EastNFCValidxSN;
                        AxisFifoErrorxDP.EastCounterxD <= AxisFifoErrorxDN.EastCounterxD;
                    end if;
                end process EastUpdateRegxP;

                SouthUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                            BackPressureRstxR.SouthxR) is
                begin  -- process SouthUpdateRegxP
                    if BackPressureRstxR.SouthxR = '1' then
                        SouthProgFullxSP                <= '0';
                        SouthProgEmptyxSP               <= '0';
                        SouthNFCStatexDP                <= C_NFC_IDLE;
                        SouthNFCDataxDP                 <= (others => '0');
                        SouthNFCValidxSP                <= '0';
                        AxisFifoErrorxDP.SouthCounterxD <= (others => '0');
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        SouthProgFullxSP                <= SouthProgFullxSN;
                        SouthProgEmptyxSP               <= SouthProgEmptyxSN;
                        SouthNFCStatexDP                <= SouthNFCStatexDN;
                        SouthNFCDataxDP                 <= SouthNFCDataxDN;
                        SouthNFCValidxSP                <= SouthNFCValidxSN;
                        AxisFifoErrorxDP.SouthCounterxD <= AxisFifoErrorxDN.SouthCounterxD;
                    end if;
                end process SouthUpdateRegxP;

                WestUpdateRegxP : process (AuroraClkOutxC.UserClkxC,
                                           BackPressureRstxR.WestxR) is
                begin  -- process WestUpdateRegxP
                    if BackPressureRstxR.WestxR = '1' then
                        WestProgFullxSP                <= '0';
                        WestProgEmptyxSP               <= '0';
                        WestNFCStatexDP                <= C_NFC_IDLE;
                        WestNFCDataxDP                 <= (others => '0');
                        WestNFCValidxSP                <= '0';
                        AxisFifoErrorxDP.WestCounterxD <= (others => '0');
                    elsif rising_edge(AuroraClkOutxC.UserClkxC) then
                        WestProgFullxSP                <= WestProgFullxSN;
                        WestProgEmptyxSP               <= WestProgEmptyxSN;
                        WestNFCStatexDP                <= WestNFCStatexDN;
                        WestNFCDataxDP                 <= WestNFCDataxDN;
                        WestNFCValidxSP                <= WestNFCValidxSN;
                        AxisFifoErrorxDP.WestCounterxD <= AxisFifoErrorxDN.WestCounterxD;
                    end if;
                end process WestUpdateRegxP;

                NorthErrorCounterxP : process (AxisFifoErrorxDP.NorthCounterxD,
                                               NorthFifoRXS2MxD.ReadyxS,
                                               NorthProgFullxSN,
                                               NorthProgFullxSP) is
                begin  -- process NorthErrorCounterxP
                    -- Default values
                    AxisFifoErrorxDN.NorthCounterxD <= AxisFifoErrorxDP.NorthCounterxD;

                    if ((NorthProgFullxSP = '0' and NorthProgFullxSN = '0') or
                        (NorthProgFullxSP = '0' and NorthProgFullxSN = '1')) and
                        NorthFifoRXS2MxD.ReadyxS = '0' then
                        AxisFifoErrorxDN.NorthCounterxD <= AxisFifoErrorxDP.NorthCounterxD + 1;
                    end if;
                end process NorthErrorCounterxP;

                NorthNFCStatexP : process (NorthNFCDataxDP, NorthNFCStatexDP,
                                           NorthNFCValidxSP, NorthProgEmptyxSN,
                                           NorthProgEmptyxSP, NorthProgFullxSN,
                                           NorthProgFullxSP,
                                           NorthTXNFCS2MxD.ReadyxS) is
                begin  -- process NorthNFCStatexP
                    -- Default values
                    NorthNFCStatexDN <= NorthNFCStatexDP;
                    NorthNFCDataxDN  <= NorthNFCDataxDP;
                    NorthNFCValidxSN <= NorthNFCValidxSP;

                    case NorthNFCStatexDP is
                        when C_NFC_IDLE =>
                                if (NorthProgFullxSP = '0' and NorthProgFullxSN = '0') then
                                    NorthNFCStatexDN <= C_NFC_IS_XON;
                                else
                                    NorthNFCDataxDN  <= C_NFC_XOFF;
                                    NorthNFCValidxSN <= '1';
                                    NorthNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XON =>
                                if (NorthProgFullxSP = '0' and NorthProgFullxSN = '1') or
                                    (NorthProgFullxSP = '1' and NorthProgFullxSN = '1') then
                                    NorthNFCDataxDN  <= C_NFC_XOFF;
                                    NorthNFCValidxSN <= '1';
                                    NorthNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XOFF =>
                                if ((NorthProgEmptyxSP = '0' and NorthProgEmptyxSN = '1') or
                                    (NorthProgEmptyxSP = '1' and NorthProgEmptyxSN = '1')) and
                                    (NorthProgFullxSP = '0' and NorthProgFullxSN = '0') then
                                    NorthNFCDataxDN  <= C_NFC_XON;
                                    NorthNFCValidxSN <= '1';
                                    NorthNFCStatexDN <= C_NFC_SEND_XON;
                                end if;

                        when C_NFC_SEND_XON =>
                                if NorthTXNFCS2MxD.ReadyxS = '1' then
                                    NorthNFCValidxSN <= '0';
                                    NorthNFCStatexDN <= C_NFC_IS_XON;
                                end if;

                        when C_NFC_SEND_XOFF =>
                                if NorthTXNFCS2MxD.ReadyxS = '1' then
                                    NorthNFCValidxSN <= '0';
                                    NorthNFCStatexDN <= C_NFC_IS_XOFF;
                                end if;

                        when others => null;
                    end case;

                end process NorthNFCStatexP;

                EastErrorCounterxP : process (AxisFifoErrorxDP.EastCounterxD,
                                              EastFifoRXS2MxD.ReadyxS,
                                              EastProgFullxSN, EastProgFullxSP) is
                begin  -- process EastErrorCounterxP
                    -- Default values
                    AxisFifoErrorxDN.EastCounterxD <= AxisFifoErrorxDP.EastCounterxD;

                    if ((EastProgFullxSP = '0' and EastProgFullxSN = '0') or
                        (EastProgFullxSP = '0' and EastProgFullxSN = '1')) and
                        EastFifoRXS2MxD.ReadyxS = '0' then
                        AxisFifoErrorxDN.EastCounterxD <= AxisFifoErrorxDP.EastCounterxD + 1;
                    end if;
                end process EastErrorCounterxP;

                EastNFCStatexP : process (EastNFCDataxDP, EastNFCStatexDP,
                                          EastNFCValidxSP, EastProgEmptyxSN,
                                          EastProgEmptyxSP, EastProgFullxSN,
                                          EastProgFullxSP,
                                          EastTXNFCS2MxD.ReadyxS) is
                begin  -- process EastNFCStatexP
                    -- Default values
                    EastNFCStatexDN <= EastNFCStatexDP;
                    EastNFCDataxDN  <= EastNFCDataxDP;
                    EastNFCValidxSN <= EastNFCValidxSP;

                    case EastNFCStatexDP is
                        when C_NFC_IDLE =>
                                if (EastProgFullxSP = '0' and EastProgFullxSN = '0') then
                                    EastNFCStatexDN <= C_NFC_IS_XON;
                                else
                                    EastNFCDataxDN  <= C_NFC_XOFF;
                                    EastNFCValidxSN <= '1';
                                    EastNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XON =>
                                if (EastProgFullxSP = '0' and EastProgFullxSN = '1') or
                                    (EastProgFullxSP = '1' and EastProgFullxSN = '1') then
                                    EastNFCDataxDN  <= C_NFC_XOFF;
                                    EastNFCValidxSN <= '1';
                                    EastNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XOFF =>
                                if ((EastProgEmptyxSP = '0' and EastProgEmptyxSN = '1') or
                                    (EastProgEmptyxSP = '1' and EastProgEmptyxSN = '1')) and
                                    (EastProgFullxSP = '0' and EastProgFullxSN = '0') then
                                    EastNFCDataxDN  <= C_NFC_XON;
                                    EastNFCValidxSN <= '1';
                                    EastNFCStatexDN <= C_NFC_SEND_XON;
                                end if;

                        when C_NFC_SEND_XON =>
                                if EastTXNFCS2MxD.ReadyxS = '1' then
                                    EastNFCValidxSN <= '0';
                                    EastNFCStatexDN <= C_NFC_IS_XON;
                                end if;

                        when C_NFC_SEND_XOFF =>
                                if EastTXNFCS2MxD.ReadyxS = '1' then
                                    EastNFCValidxSN <= '0';
                                    EastNFCStatexDN <= C_NFC_IS_XOFF;
                                end if;

                        when others => null;
                    end case;

                end process EastNFCStatexP;

                SouthErrorCounterxP : process (AxisFifoErrorxDP.SouthCounterxD,
                                               SouthFifoRXS2MxD.ReadyxS,
                                               SouthProgFullxSN,
                                               SouthProgFullxSP) is
                begin  -- process SouthErrorCounterxP
                    -- Default values
                    AxisFifoErrorxDN.SouthCounterxD <= AxisFifoErrorxDP.SouthCounterxD;

                    if ((SouthProgFullxSP = '0' and SouthProgFullxSN = '0') or
                        (SouthProgFullxSP = '0' and SouthProgFullxSN = '1')) and
                        SouthFifoRXS2MxD.ReadyxS = '0' then
                        AxisFifoErrorxDN.SouthCounterxD <= AxisFifoErrorxDP.SouthCounterxD + 1;
                    end if;
                end process SouthErrorCounterxP;

                SouthNFCStatexP : process (SouthNFCDataxDP, SouthNFCStatexDP,
                                           SouthNFCValidxSP, SouthProgEmptyxSN,
                                           SouthProgEmptyxSP, SouthProgFullxSN,
                                           SouthProgFullxSP,
                                           SouthTXNFCS2MxD.ReadyxS) is
                begin  -- process SouthNFCStatexP
                    -- Default values
                    SouthNFCStatexDN <= SouthNFCStatexDP;
                    SouthNFCDataxDN  <= SouthNFCDataxDP;
                    SouthNFCValidxSN <= SouthNFCValidxSP;

                    case SouthNFCStatexDP is
                        when C_NFC_IDLE =>
                                if (SouthProgFullxSP = '0' and SouthProgFullxSN = '0') then
                                    SouthNFCStatexDN <= C_NFC_IS_XON;
                                else
                                    SouthNFCDataxDN  <= C_NFC_XOFF;
                                    SouthNFCValidxSN <= '1';
                                    SouthNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XON =>
                                if (SouthProgFullxSP = '0' and SouthProgFullxSN = '1') or
                                    (SouthProgFullxSP = '1' and SouthProgFullxSN = '1') then
                                    SouthNFCDataxDN  <= C_NFC_XOFF;
                                    SouthNFCValidxSN <= '1';
                                    SouthNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XOFF =>
                                if ((SouthProgEmptyxSP = '0' and SouthProgEmptyxSN = '1') or
                                    (SouthProgEmptyxSP = '1' and SouthProgEmptyxSN = '1')) and
                                    (SouthProgFullxSP = '0' and SouthProgFullxSN = '0') then
                                    SouthNFCDataxDN  <= C_NFC_XON;
                                    SouthNFCValidxSN <= '1';
                                    SouthNFCStatexDN <= C_NFC_SEND_XON;
                                end if;

                        when C_NFC_SEND_XON =>
                                if SouthTXNFCS2MxD.ReadyxS = '1' then
                                    SouthNFCValidxSN <= '0';
                                    SouthNFCStatexDN <= C_NFC_IS_XON;
                                end if;

                        when C_NFC_SEND_XOFF =>
                                if SouthTXNFCS2MxD.ReadyxS = '1' then
                                    SouthNFCValidxSN <= '0';
                                    SouthNFCStatexDN <= C_NFC_IS_XOFF;
                                end if;

                        when others => null;
                    end case;

                end process SouthNFCStatexP;

                WestErrorCounterxP : process (AxisFifoErrorxDP.WestCounterxD,
                                              WestFifoRXS2MxD.ReadyxS,
                                              WestProgFullxSN, WestProgFullxSP) is
                begin  -- process WestErrorCounterxP
                    -- Default values
                    AxisFifoErrorxDN.WestCounterxD <= AxisFifoErrorxDP.WestCounterxD;

                    if ((WestProgFullxSP = '0' and WestProgFullxSN = '0') or
                        (WestProgFullxSP = '0' and WestProgFullxSN = '1')) and
                        WestFifoRXS2MxD.ReadyxS = '0' then
                        AxisFifoErrorxDN.WestCounterxD <= AxisFifoErrorxDP.WestCounterxD + 1;
                    end if;
                end process WestErrorCounterxP;

                WestNFCStatexP : process (WestNFCDataxDP, WestNFCStatexDP,
                                          WestNFCValidxSP, WestProgEmptyxSN,
                                          WestProgEmptyxSP, WestProgFullxSN,
                                          WestProgFullxSP,
                                          WestTXNFCS2MxD.ReadyxS) is
                begin  -- process WestNFCStatexP
                    -- Default values
                    WestNFCStatexDN <= WestNFCStatexDP;
                    WestNFCDataxDN  <= WestNFCDataxDP;
                    WestNFCValidxSN <= WestNFCValidxSP;

                    case WestNFCStatexDP is
                        when C_NFC_IDLE =>
                                if (WestProgFullxSP = '0' and WestProgFullxSN = '0') then
                                    WestNFCStatexDN <= C_NFC_IS_XON;
                                else
                                    WestNFCDataxDN  <= C_NFC_XOFF;
                                    WestNFCValidxSN <= '1';
                                    WestNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XON =>
                                if (WestProgFullxSP = '0' and WestProgFullxSN = '1') or
                                    (WestProgFullxSP = '1' and WestProgFullxSN = '1') then
                                    WestNFCDataxDN  <= C_NFC_XOFF;
                                    WestNFCValidxSN <= '1';
                                    WestNFCStatexDN <= C_NFC_SEND_XOFF;
                                end if;

                        when C_NFC_IS_XOFF =>
                                if ((WestProgEmptyxSP = '0' and WestProgEmptyxSN = '1') or
                                    (WestProgEmptyxSP = '1' and WestProgEmptyxSN = '1')) and
                                    (WestProgFullxSP = '0' and WestProgFullxSN = '0') then
                                    WestNFCDataxDN  <= C_NFC_XON;
                                    WestNFCValidxSN <= '1';
                                    WestNFCStatexDN <= C_NFC_SEND_XON;
                                end if;

                        when C_NFC_SEND_XON =>
                                if WestTXNFCS2MxD.ReadyxS = '1' then
                                    WestNFCValidxSN <= '0';
                                    WestNFCStatexDN <= C_NFC_IS_XON;
                                end if;

                        when C_NFC_SEND_XOFF =>
                                if WestTXNFCS2MxD.ReadyxS = '1' then
                                    WestNFCValidxSN <= '0';
                                    WestNFCStatexDN <= C_NFC_IS_XOFF;
                                end if;

                        when others => null;
                    end case;

                end process WestNFCStatexP;

            end block RXBackPressurexB;

        end generate RXFifoModexG;

    end block RXModexB;

end arch;
