----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_fifo_double_register.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_fifo_double_register.
--
-- Last update: 2020-02-21
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
-- use ieee.math_real.uniform;
-- use ieee.math_real.floor;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.aurora_axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

entity tb_scalp_fifo_double_register is

end entity tb_scalp_fifo_double_register;

architecture testbench of tb_scalp_fifo_double_register is

    constant C_CLK_PERIOD : time := 10 ns;

    component scalp_fifo_double_register is
        port (
            -- System Clock and Reset
            SysClkxCI      : in  std_ulogic;
            SysRstxRNAI    : in  std_ulogic;
            -- Axis Ports (Slave Side)
            Axism2sLinkxDI : in  t_axi4m2s;
            Axiss2mLinkxDO : out t_axi4s2m;
            -- Axis Ports (Master Side)
            Axism2sLinkxDO : out t_axi4m2s;
            Axiss2mLinkxDI : in  t_axi4s2m);
    end component scalp_fifo_double_register;

    signal SysClkxC         : std_ulogic := '0';
    signal SysRstxRNA       : std_ulogic := '0';
    -- End of Simulation
    signal EndSimxS         : boolean    := false;
    -- Others
    signal Axism2sLinkInxD  : t_axi4m2s  := C_NO_AXI4_M2S;
    signal Axiss2mLinkOutxD : t_axi4s2m  := C_NO_AXI4_S2M;
    signal Axism2sLinkOutxD : t_axi4m2s  := C_NO_AXI4_M2S;
    signal Axiss2mLinkInxD  : t_axi4s2m  := C_NO_AXI4_S2M;
    -- Counter
    signal CounterxDP       : integer    := 0;
    signal CounterxDN       : integer    := 0;

begin  -- architecture testbench

    AsyncStatementsxB : block is
    begin  -- block AsyncStatementsxB
        SysClkxAS : SysClkxC               <= not SysClkxC after (C_CLK_PERIOD / 2);
        SysRstxAS : SysRstxRNA             <= '1'          after (C_CLK_PERIOD * 10);
        DataxAS   : Axism2sLinkInxD.DataxD <= std_ulogic_vector(to_unsigned(CounterxDP, 32));
    end block AsyncStatementsxB;

    EndSimxP : process is
    begin  -- process EndSimxP
        if EndSimxS = true then
            assert false report "End of simulation!" severity failure;
        end if;

        wait until rising_edge(SysClkxC);
    end process EndSimxP;

    ScalpFifoDoubleRegisterxI : entity work.scalp_fifo_double_register
        port map (
            -- System Clock and Reset
            SysClkxCI      => SysClkxC,
            SysRstxRNAI    => SysRstxRNA,
            -- Axis Ports (Slave Side)
            Axism2sLinkxDI => Axism2sLinkInxD,
            Axiss2mLinkxDO => Axiss2mLinkOutxD,
            -- Axis Ports (Master Side)
            Axism2sLinkxDO => Axism2sLinkOutxD,
            Axiss2mLinkxDI => Axiss2mLinkInxD);

    UpdateRegxP : process (SysClkxC, SysRstxRNA) is
    begin  -- process UpdateRegxP
        if SysRstxRNA = '0' then
            CounterxDP <= 0;
        elsif rising_edge(SysClkxC) then
            CounterxDP <= CounterxDN;
        end if;
    end process UpdateRegxP;

    CounterxP : process (Axism2sLinkInxD.ValidxS, Axiss2mLinkOutxD.ReadyxS,
                         CounterxDP) is
    begin  -- process CounterxP
        CounterxDN <= CounterxDP;

        if Axiss2mLinkOutxD.ReadyxS = '1' then
            if Axism2sLinkInxD.ValidxS = '1' then
                CounterxDN <= CounterxDP + 1;
            end if;
        end if;
    end process CounterxP;

    ValidxP : process is
    begin  -- process ValidxP
        Axism2sLinkInxD.ValidxS <= '0';
        wait until (SysRstxRNA = '1');
        Axism2sLinkInxD.ValidxS <= '1';

        for i in 0 to 30 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axism2sLinkInxD.ValidxS <= '0';

        for i in 0 to 6 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axism2sLinkInxD.ValidxS <= '1';

        for i in 0 to 14 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axism2sLinkInxD.ValidxS <= '0';

        for i in 0 to 10 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axism2sLinkInxD.ValidxS <= '1';

        wait;
    end process ValidxP;

    ReadyxP : process is
    begin  -- process ReadyxP
        Axiss2mLinkInxD.ReadyxS <= '0';
        wait until (SysRstxRNA = '1');
        Axiss2mLinkInxD.ReadyxS <= '1';
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        for i in 0 to 10 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axiss2mLinkInxD.ReadyxS <= '0';

        for i in 0 to 5 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axiss2mLinkInxD.ReadyxS <= '1';

        for i in 0 to 40 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axiss2mLinkInxD.ReadyxS <= '0';

        for i in 0 to 5 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        for i in 0 to 20 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        Axiss2mLinkInxD.ReadyxS <= '1';

        for i in 0 to 5 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        for i in 0 to 10 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        wait;
    end process ReadyxP;

    TestbenchxP : process is
    begin  -- process TestbenchxP
        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        for i in 0 to 150 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        EndSimxS <= true;
    end process TestbenchxP;

end architecture testbench;
