----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Quentin Berthet <quentin.berthet@hesge.ch
--
-- Module Name: scalp_hl2_bram
-- Target Device: hepia-cores.ch:scalp_node:part0:0.1 xc7z015clg485-2
-- Tool version: 2019.2
-- Description: Instanciates simple dual port BRAM bank generic in size.
--
-- Last update: 2020/12/17 12:09:55
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library xpm;
use xpm.vcomponents.all;

library xil_defaultlib;
use xil_defaultlib.scalp_hl2_pkg.all;

entity scalp_hl2_bram is
    generic (
        G_PORT_COUNT : integer := 1
    );
    port (
        i_clk  : in std_logic;
        i_nrst : in std_logic;

        -- AXI Ports Data bus
        o_axi_data : out std_logic_vector(31 downto 0);
        i_axi_addr : in std_logic_vector(16 downto 0);
        i_axi_data : in std_logic_vector(31 downto 0);
        i_axi_en   : in std_logic;
        i_axi_we   : in std_logic_vector(3 downto 0);

        -- DMX Ports Data bus
        o_dmx_data : out std_logic_vector((8 * G_PORT_COUNT) - 1 downto 0);
        i_dmx_addr : in std_logic_vector(C_REG_CHAN_COUNT_SIZE - 1 downto 0);
        i_dmx_rden : in std_logic
    );
end scalp_hl2_bram;

architecture RTL of scalp_hl2_bram is

    component blk_mem_gen_0
        port (
            clka      : in std_logic;
            rsta      : in std_logic;
            ena       : in std_logic;
            wea       : in std_logic_vector(3 downto 0);
            addra     : in std_logic_vector(9 downto 0);
            dina      : in std_logic_vector(31 downto 0);
            douta     : out std_logic_vector(31 downto 0);
            clkb      : in std_logic;
            rstb      : in std_logic;
            enb       : in std_logic;
            web       : in std_logic_vector(0 downto 0);
            addrb     : in std_logic_vector(11 downto 0);
            dinb      : in std_logic_vector(7 downto 0);
            doutb     : out std_logic_vector(7 downto 0);
            rsta_busy : out std_logic;
            rstb_busy : out std_logic
        );
    end component;

    signal s_rst : std_logic;

    signal s_bram_sel  : std_logic_vector(6 downto 0);
    signal s_bram_addr : std_logic_vector(9 downto 0);

    signal s_en      : std_logic;
    signal s_bram_en : std_logic_vector(G_PORT_COUNT - 1 downto 0);

    type s_axi_data_out_t is array (0 to G_PORT_COUNT - 1) of std_logic_vector(31 downto 0);
    signal s_axi_data_out : s_axi_data_out_t;

begin

    -- Reset logic
    s_rst <= not i_nrst;

    -- Split AXI address into BRAM select and BRAM address
    s_bram_sel  <= i_axi_addr(16 downto 10);
    s_bram_addr <= i_axi_addr(9 downto 0);

    s_en <= i_axi_en;

    -- Bram selection
    -- Alternate to test:
    -- output <= (others => '0'); -- default
    -- output(to_integer(input)) <= '1';
    BRAM_SEL : process (i_nrst, s_bram_sel, s_en)
    begin
        if i_nrst = '0' then
            s_bram_en <= (others => '0');
        else
            for i in 0 to G_PORT_COUNT - 1 loop
                if i = to_integer(unsigned(s_bram_sel)) then
                    s_bram_en(i) <= s_en;
                else
                    s_bram_en(i) <= '0';
                end if;
            end loop;
        end if;
    end process BRAM_SEL;

    -- Data out selection
    DOUT_SEL : process (s_bram_sel, s_axi_data_out)
    begin
        o_axi_data <= s_axi_data_out(to_integer(unsigned(s_bram_sel)));
    end process DOUT_SEL;

    -- BRAM instances
    RAM_GEN : for i in 0 to G_PORT_COUNT - 1 generate
    begin

        blk_mem_gen_i : blk_mem_gen_0
        port map(
            clka      => i_clk,
            rsta      => s_rst,
            ena       => s_bram_en(i),
            wea       => i_axi_we,
            addra     => s_bram_addr,
            dina      => i_axi_data,
            douta     => s_axi_data_out(i),
            clkb      => i_clk,
            rstb      => s_rst,
            enb       => i_dmx_rden,
            web       => "0",
            addrb     => i_dmx_addr,
            dinb => (others => '0'),
            doutb     => o_dmx_data(((i * 8) + 7) downto i * 8),
            rsta_busy => open,
            rstb_busy => open
        );

    end generate RAM_GEN;

end RTL;