----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Quentin Berthet <quentin.berthet@hesge.ch
--
-- Module Name: scalp_hl2_ctrl
-- Target Device: hepia-cores.ch:scalp_node:part0:0.1 xc7z015clg485-2
-- Tool version: 2019.2
-- Description: Implement serialization state-machine: DMX-512 protocol
--
-- Last update: 2020/12/21 17:31:41
--
---------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.all;

library xil_defaultlib;
use xil_defaultlib.scalp_hl2_pkg.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity scalp_hl2_ctrl is
    generic (
        G_PORT_COUNT : integer := 1
    );
    port (
        i_clk  : in std_logic;
        i_nrst : in std_logic;

        -- Config registers
        ir_mab_clk    : in std_logic_vector(C_REG_MAB_CLK_SIZE - 1 downto 0);
        ir_break_clk  : in std_logic_vector(C_REG_BREAK_CLK_SIZE - 1 downto 0);
        ir_bit_clk    : in std_logic_vector(C_REG_BIT_CLK_SIZE - 1 downto 0);
        ir_chan_count : in std_logic_vector(C_REG_CHAN_COUNT_SIZE - 1 downto 0);
        ir_global_en  : in std_logic_vector(C_REG_GLOBAL_EN_SIZE - 1 downto 0);
        ir_sof_value  : in std_logic_vector((C_REG_SOF_VALUE_SIZE * G_PORT_COUNT) - 1 downto 0);
        ir_idl_value  : in std_logic_vector((C_REG_IDL_SIZE * G_PORT_COUNT) - 1 downto 0);
        ir_chan_en    : in std_logic_vector((C_REG_EN_SIZE * G_PORT_COUNT) - 1 downto 0);

        -- Data bus
        i_data : in std_logic_vector((8 * G_PORT_COUNT) - 1 downto 0);
        o_addr : out std_logic_vector(C_REG_CHAN_COUNT_SIZE - 1 downto 0);
        o_rden : out std_logic;

        -- Uart output vector
        o_uart : out std_logic_vector(G_PORT_COUNT - 1 downto 0)
    );
end scalp_hl2_ctrl;

architecture Behavioral of scalp_hl2_ctrl is

    type t_SM_Main is (
        s_Init,
        s_Break,
        s_MarkAfterBreak,
        s_StartOfFrame,
        s_WaitStartOfFrame,
        s_DataFrame,
        s_WaitDataFrame
    );

    signal r_SM_Main : t_SM_Main;

    signal s_uart_done   : std_logic;
    signal s_uart_strobe : std_logic;
    signal s_uart        : std_logic_vector(G_PORT_COUNT - 1 downto 0);
    signal r_tx_data     : std_logic_vector((G_PORT_COUNT * 8) - 1 downto 0);
    signal r_Clk_Count   : unsigned(C_REG_BREAK_CLK_SIZE - 1 downto 0);
    signal r_data_addr   : unsigned(C_REG_CHAN_COUNT_SIZE - 1 downto 0);

    component scalp_hl2_uart is
        generic (
            G_PORT_COUNT : integer
        );
        port (
            i_clk        : in std_logic;
            ir_bit_clk   : in std_logic_vector(C_REG_BIT_CLK_SIZE - 1 downto 0);
            i_data_valid : in std_logic;
            i_data       : in std_logic_vector((8 * G_PORT_COUNT) - 1 downto 0);
            o_active     : out std_logic;
            o_uart       : out std_logic_vector(G_PORT_COUNT - 1 downto 0);
            o_done       : out std_logic
        );
    end component scalp_hl2_uart;

begin

    -- DMX512 State machine
    p_DMX512 : process (i_clk)
    begin
        if rising_edge(i_Clk) then
            if i_nrst = '0' or ir_global_en = "0" then
                -- Init out line and counters
                r_Clk_Count <= (others => '0');
                r_data_addr <= (others => '0');
                r_SM_Main   <= s_Init;
            else
                case r_SM_Main is
                    when s_Init =>
                        -- Init out line and counters
                        r_Clk_Count <= (others => '0');
                        r_data_addr <= (others => '0');
                        -- go to s_Break
                        r_SM_Main <= s_Break;
                    when s_Break =>
                        -- Set out line low
                        --   Done in the mux at the end of file

                        -- wait ir_break_clk clock cycles
                        if r_Clk_Count < unsigned(ir_break_clk) - 1 then
                            r_Clk_Count <= r_Clk_Count + 1;
                            r_SM_Main   <= s_Break;
                        else
                            r_Clk_Count <= (others => '0');
                            r_SM_Main   <= s_MarkAfterBreak;
                        end if;

                        -- go to s_MarkAfterBreak
                    when s_MarkAfterBreak =>
                        -- Set out line high
                        --   Done in the mux at the end of file

                        -- Preload SOF value, 0 by default
                        r_tx_data   <= ir_sof_value;
                        r_data_addr <= (others => '0');

                        -- wait ir_mab_clk clock cycles
                        if r_Clk_Count < unsigned(ir_mab_clk) - 3 then
                            r_Clk_Count <= r_Clk_Count + 1;
                            r_SM_Main   <= s_MarkAfterBreak;
                        else
                            r_Clk_Count <= (others => '0');
                            r_SM_Main   <= s_StartOfFrame;
                        end if;

                        -- go to s_StartOfFrame
                    when s_StartOfFrame =>
                        -- Send null byte

                        -- go to s_WaitStartOfFrame
                        r_SM_Main <= s_WaitStartOfFrame;

                    when s_WaitStartOfFrame =>
                        if s_uart_done = '1' then
                            r_SM_Main <= s_DataFrame;
                            r_tx_data <= i_data;
                        else
                            r_SM_Main <= s_WaitStartOfFrame;
                        end if;

                    when s_DataFrame =>
                        r_SM_Main   <= s_WaitDataFrame;
                        r_data_addr <= r_data_addr + 1;

                    when s_WaitDataFrame =>
                        if s_uart_done = '1' then
                            if r_data_addr < unsigned(ir_chan_count) then
                                r_SM_Main <= s_DataFrame;
                                r_tx_data <= i_data;
                            else
                                r_SM_Main <= s_Break;
                            end if;
                        else
                            r_SM_Main <= s_WaitDataFrame;
                        end if;

                    when others =>
                        r_SM_Main <= s_Init;

                end case;
            end if;
        end if;
    end process p_DMX512;

    with r_SM_Main select s_uart_strobe <=
        '1' when s_StartOfFrame,
        '1' when s_DataFrame,
        '0' when others;

    o_addr <= std_logic_vector(r_data_addr);

    -- Memory always selected for now, see if FSM needs to control it
    o_rden <= '1';

    -- Uart instance
    scalp_hl2_uart_i : scalp_hl2_uart
    generic map(
        G_PORT_COUNT => G_PORT_COUNT
    )
    port map(
        i_clk        => i_clk,
        ir_bit_clk   => ir_bit_clk,
        i_data_valid => s_uart_strobe,
        i_data       => r_tx_data,
        o_active     => open,
        o_uart       => s_uart,
        o_done       => s_uart_done
    );

    -- DMX output signal control
    --with r_SM_Main select o_dmx <= (others => '0') when s_Init,
    --                               (others => '0') when s_Break,
    --                               (others => '1') when s_MarkAfterBreak,
    --                               s_uartdmx when others;

    -- DMX512 State machine
    p_DMX_OUT : process (r_SM_Main, ir_chan_en, ir_idl_value, s_uart)
    begin
        for I in 0 to G_PORT_COUNT - 1 loop
            if ir_chan_en(I) = '0' then
                o_uart(I) <= ir_idl_value(I);
            else
                case r_SM_Main is
                    when s_Init =>
                        o_uart(I) <= '0';
                    when s_Break =>
                        o_uart(I) <= '0';
                    when s_MarkAfterBreak =>
                        o_uart(I) <= '1';
                    when others =>
                        o_uart(I) <= s_uart(I);
                end case;
            end if;
        end loop;
    end process;

end Behavioral;