----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Quentin Berthet <quentin.berthet@hesge.ch
--
-- Module Name: scalp_hl2_uart
-- Target Device: hepia-cores.ch:scalp_node:part0:0.1 xc7z015clg485-2
-- Tool version: 2019.2
-- Description: Simple unidirectional UART, with configurable port count.
--              Implemented with DMX-512 in mind..
--
-- Last update: 2020/12/21 16:31:35
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library xil_defaultlib;
use xil_defaultlib.scalp_hl2_pkg.all;

entity scalp_hl2_uart is
    generic (
        G_PORT_COUNT : integer := 1
    );
    port (
        i_clk        : in std_logic;
        ir_bit_clk   : in std_logic_vector(C_REG_BIT_CLK_SIZE - 1 downto 0);
        i_data_valid : in std_logic;
        i_data       : in std_logic_vector((8 * G_PORT_COUNT) - 1 downto 0);
        o_active     : out std_logic;
        o_uart       : out std_logic_vector(G_PORT_COUNT - 1 downto 0);
        o_done       : out std_logic
    );
end scalp_hl2_uart;
architecture RTL of scalp_hl2_uart is

    type t_SM_Main is (
        s_Idle,
        s_TX_Start_Bit,
        s_TX_Data_Bits,
        s_TX_Stop_Bit1,
        s_TX_Stop_Bit2,
        s_Cleanup
    );

    signal r_SM_Main : t_SM_Main := s_Idle;

    signal r_clk_counter : unsigned(C_REG_BIT_CLK_SIZE - 1 downto 0);
    signal r_bit_index   : integer range 0 to 7;
    signal r_data        : std_logic_vector((8 * G_PORT_COUNT) - 1 downto 0);
    signal r_done        : std_logic;

begin

    p_UART_TX : process (i_clk)
    begin
        if rising_edge(i_clk) then

            case r_SM_Main is

                when s_Idle =>
                    o_active      <= '0';
                    o_uart        <= (others => '1'); -- Drive Line High for Idle
                    r_done        <= '0';
                    r_clk_counter <= (others => '0');
                    r_bit_index   <= 0;

                    if i_data_valid = '1' then
                        r_data    <= i_data;
                        r_SM_Main <= s_TX_Start_Bit;
                    else
                        r_SM_Main <= s_Idle;
                    end if;

                    -- Send out Start Bit. Start bit = 0
                when s_TX_Start_Bit =>
                    o_active <= '1';
                    o_uart   <= (others => '0');

                    -- Wait ir_bit_clk clock cycles for start bit to finish
                    if r_clk_counter < unsigned(ir_bit_clk) - 1 then
                        r_clk_counter <= r_clk_counter + 1;
                        r_SM_Main     <= s_TX_Start_Bit;
                    else
                        r_clk_counter <= (others => '0');
                        r_SM_Main     <= s_TX_Data_Bits;
                    end if;
                    -- Wait ir_bit_clk clock cycles for data bits to finish
                when s_TX_Data_Bits =>
                    for I in 0 to G_PORT_COUNT - 1 loop
                        o_uart(I) <= r_data(r_bit_index + (8 * I));
                    end loop;

                    if r_clk_counter < unsigned(ir_bit_clk) - 1 then
                        r_clk_counter <= r_clk_counter + 1;
                        r_SM_Main     <= s_TX_Data_Bits;
                    else
                        r_clk_counter <= (others => '0');

                        -- Check if we have sent out all bits
                        if r_bit_index < 7 then
                            r_bit_index <= r_bit_index + 1;
                            r_SM_Main   <= s_TX_Data_Bits;
                        else
                            r_bit_index <= 0;
                            r_SM_Main   <= s_TX_Stop_Bit1;
                        end if;
                    end if;
                    -- Send out first stop bit
                when s_TX_Stop_Bit1 =>
                    o_uart <= (others   => '1');

                    -- Wait ir_bit_clk clock cycles for Stop bit to finish
                    if r_clk_counter < unsigned(ir_bit_clk) - 1 then
                        r_clk_counter <= r_clk_counter + 1;
                        r_SM_Main     <= s_TX_Stop_Bit1;
                    else
                        r_clk_counter <= (others => '0');
                        r_SM_Main     <= s_TX_Stop_Bit2;
                    end if;
                    -- Send out second stop bit
                when s_TX_Stop_Bit2 =>
                    o_uart <= (others   => '1');

                    -- Wait ir_bit_clk clock cycles for Stop bit to finish
                    if r_clk_counter < unsigned(ir_bit_clk) - 2 then
                        r_clk_counter <= r_clk_counter + 1;
                        r_SM_Main     <= s_TX_Stop_Bit2;
                    else
                        r_clk_counter <= (others => '0');
                        r_SM_Main     <= s_Cleanup;
                    end if;

                    -- Stay here 1 clock
                when s_Cleanup =>
                    o_active  <= '0';
                    r_done    <= '1';
                    r_SM_Main <= s_Idle;
                when others =>
                    r_SM_Main <= s_Idle;

            end case;
        end if;
    end process p_UART_TX;

    o_done <= r_done;

end RTL;